@@ -10,12 +10,13 @@ from datetime import datetime
 from enum import Enum
 from typing import Callable, Dict, Iterator, List, Optional, Tuple, Union
 
-from maggma.core.validator import Validator
-from maggma.utils import LU_KEY_ISOFORMAT
 from monty.dev import deprecated
 from monty.json import MontyDecoder, MSONable
 from pydash import get, has, identity
 
+from maggma.core.validator import Validator
+from maggma.utils import LU_KEY_ISOFORMAT
+
 
 class Sort(Enum):
     """ Enumeration for sorting order """
@@ -107,7 +108,7 @@ class Store(MSONable, metaclass=ABCMeta):
         self,
         criteria: Optional[Dict] = None,
         properties: Union[Dict, List, None] = None,
-        sort: Optional[Dict[str, Sort]] = None,
+        sort: Optional[Dict[str, Union[Sort, int]]] = None,
         skip: int = 0,
         limit: int = 0,
     ) -> Iterator[Dict]:
@@ -117,7 +118,8 @@ class Store(MSONable, metaclass=ABCMeta):
         Args:
             criteria: PyMongo filter for documents to search in
             properties: properties to return in grouped documents
-            sort: Dictionary of sort order for fields
+            sort: Dictionary of sort order for fields. Keys are field names and
+                values are 1 for ascending or -1 for descending.
             skip: number documents to skip
             limit: limit on total number of documents returned
         """
@@ -154,7 +156,7 @@ class Store(MSONable, metaclass=ABCMeta):
         keys: Union[List[str], str],
         criteria: Optional[Dict] = None,
         properties: Union[Dict, List, None] = None,
-        sort: Optional[Dict[str, Sort]] = None,
+        sort: Optional[Dict[str, Union[Sort, int]]] = None,
         skip: int = 0,
         limit: int = 0,
     ) -> Iterator[Tuple[Dict, List[Dict]]]:
@@ -166,7 +168,8 @@ class Store(MSONable, metaclass=ABCMeta):
             keys: fields to group documents
             criteria: PyMongo filter for documents to search in
             properties: properties to return in grouped documents
-            sort: Dictionary of sort order for fields
+            sort: Dictionary of sort order for fields. Keys are field names and
+                values are 1 for ascending or -1 for descending.
             skip: number documents to skip
             limit: limit on total number of documents returned
 
@@ -187,7 +190,7 @@ class Store(MSONable, metaclass=ABCMeta):
         self,
         criteria: Optional[Dict] = None,
         properties: Union[Dict, List, None] = None,
-        sort: Optional[Dict[str, Sort]] = None,
+        sort: Optional[Dict[str, Union[Sort, int]]] = None,
     ):
         """
         Queries the Store for a single document
@@ -195,7 +198,8 @@ class Store(MSONable, metaclass=ABCMeta):
         Args:
             criteria: PyMongo filter for documents to search
             properties: properties to return in the document
-            sort: Dictionary of sort order for fields
+            sort: Dictionary of sort order for fields. Keys are field names and
+                values are 1 for ascending or -1 for descending.
         """
         return next(
             self.query(criteria=criteria, properties=properties, sort=sort), None
@@ -228,7 +232,7 @@ class Store(MSONable, metaclass=ABCMeta):
         doc = next(
             self.query(
                 properties=[self.last_updated_field],
-                sort={self.last_updated_field: Sort.Descending},
+                sort={self.last_updated_field: -1},
                 limit=1,
             ),
             None,
@@ -342,6 +346,13 @@ class Store(MSONable, metaclass=ABCMeta):
         d = MontyDecoder().process_decoded(d)
         self.__init__(**d)
 
+    def __enter__(self):
+        self.connect()
+        return self
+
+    def __exit__(self, exception_type, exception_value, traceback):
+        self.close()
+
 
 class StoreError(Exception):
     """ General Store-related error """
