@@ -3,6 +3,8 @@
 Advanced Stores for connecting to AWS data
 """
 
+import threading
+import warnings
 import zlib
 from concurrent.futures import wait
 from concurrent.futures.thread import ThreadPoolExecutor
@@ -11,14 +13,13 @@ from typing import Any, Dict, Iterator, List, Optional, Tuple, Union
 import msgpack  # type: ignore
 from monty.dev import deprecated
 from monty.msgpack import default as monty_default
-from monty.msgpack import object_hook as monty_object_hook
 
 from maggma.core import Sort, Store
 from maggma.utils import grouper, to_isoformat_ceil_ms
 
 try:
-    import botocore
     import boto3
+    import botocore
     from boto3.session import Session
 except ImportError:
     boto3 = None  # type: ignore
@@ -38,7 +39,9 @@ class S3Store(Store):
         compress: bool = False,
         endpoint_url: str = None,
         sub_dir: str = None,
-        s3_workers: int = 4,
+        s3_workers: int = 1,
+        key: str = "task_id",
+        searchable_fields: List[str] = [],
         **kwargs,
     ):
         """
@@ -52,6 +55,7 @@ class S3Store(Store):
             endpoint_url: endpoint_url to allow interface to minio service
             sub_dir: (optional)  subdirectory of the s3 bucket to store the data
             s3_workers: number of concurrent S3 puts to run
+            searchable_fields: fields to keep in the index store
         """
         if boto3 is None:
             raise RuntimeError("boto3 and botocore are required for S3Store")
@@ -65,9 +69,21 @@ class S3Store(Store):
         self.s3 = None  # type: Any
         self.s3_bucket = None  # type: Any
         self.s3_workers = s3_workers
+        self.searchable_fields = searchable_fields
+
         # Force the key to be the same as the index
+        assert isinstance(
+            index.key, str
+        ), "Since we are using the key as a file name in S3, they key must be a string"
+        if key != index.key:
+            warnings.warn(
+                f'The desired S3Store key "{key}" does not match the index key "{index.key},"'
+                "the index key will be used",
+                UserWarning,
+            )
         kwargs["key"] = str(index.key)
 
+        self._thread_local = threading.local()
         super(S3Store, self).__init__(**kwargs)
 
     def name(self) -> str:
@@ -128,7 +144,7 @@ class S3Store(Store):
         self,
         criteria: Optional[Dict] = None,
         properties: Union[Dict, List, None] = None,
-        sort: Optional[Dict[str, Sort]] = None,
+        sort: Optional[Dict[str, Union[Sort, int]]] = None,
         skip: int = 0,
         limit: int = 0,
     ) -> Iterator[Dict]:
@@ -138,7 +154,8 @@ class S3Store(Store):
         Args:
             criteria: PyMongo filter for documents to search in
             properties: properties to return in grouped documents
-            sort: Dictionary of sort order for fields
+            sort: Dictionary of sort order for fields. Keys are field names and
+                values are 1 for ascending or -1 for descending.
             skip: number documents to skip
             limit: limit on total number of documents returned
         """
@@ -175,7 +192,14 @@ class S3Store(Store):
 
                 if doc.get("compression", "") == "zlib":
                     data = zlib.decompress(data)
-                yield msgpack.unpackb(data, object_hook=monty_object_hook, raw=False)
+                # requires msgpack-python to be installed to fix string encoding problem
+                # https://github.com/msgpack/msgpack/issues/121
+                # During recursion
+                # msgpack.unpackb goes as deep as possible during reconstruction
+                # MontyDecoder().process_decode only goes until it finds a from_dict
+                # as such, we cannot just use msgpack.unpackb(data, object_hook=monty_object_hook, raw=False)
+                # Should just return the unpacked object then let the user run process_decoded
+                yield msgpack.unpackb(data, raw=False)
 
     def distinct(
         self, field: str, criteria: Optional[Dict] = None, all_exist: bool = False
@@ -195,7 +219,7 @@ class S3Store(Store):
         keys: Union[List[str], str],
         criteria: Optional[Dict] = None,
         properties: Union[Dict, List, None] = None,
-        sort: Optional[Dict[str, Sort]] = None,
+        sort: Optional[Dict[str, Union[Sort, int]]] = None,
         skip: int = 0,
         limit: int = 0,
     ) -> Iterator[Tuple[Dict, List[Dict]]]:
@@ -207,7 +231,8 @@ class S3Store(Store):
             keys: fields to group documents
             criteria: PyMongo filter for documents to search in
             properties: properties to return in grouped documents
-            sort: Dictionary of sort order for fields
+            sort: Dictionary of sort order for fields. Keys are field names and
+                values are 1 for ascending or -1 for descending.
             skip: number documents to skip
             limit: limit on total number of documents returned
 
@@ -236,7 +261,12 @@ class S3Store(Store):
         """
         return self.index.ensure_index(key, unique=unique)
 
-    def update(self, docs: Union[List[Dict], Dict], key: Union[List, str, None] = None):
+    def update(
+        self,
+        docs: Union[List[Dict], Dict],
+        key: Union[List, str, None] = None,
+        additional_metadata: Union[str, List[str], None] = None,
+    ):
         """
         Update documents into the Store
 
@@ -246,27 +276,47 @@ class S3Store(Store):
                  document, can be a list of multiple fields,
                  a single field, or None if the Store's key
                  field is to be used
+            additional_metadata: field(s) to include in the s3 store's metadata
         """
-        search_docs = []
+        if not isinstance(docs, list):
+            docs = [docs]
 
-        if isinstance(key, list):
-            search_keys = key
-        elif key:
-            search_keys = [key]
+        if isinstance(key, str):
+            key = [key]
+        elif not key:
+            key = [self.key]
+
+        if additional_metadata is None:
+            additional_metadata = []
+        elif isinstance(additional_metadata, str):
+            additional_metadata = [additional_metadata]
         else:
-            search_keys = [self.key]
+            additional_metadata = list(additional_metadata)
+
         with ThreadPoolExecutor(max_workers=self.s3_workers) as pool:
             fs = {
                 pool.submit(
-                    fn=self.write_doc_to_s3, doc=itr_doc, search_keys=search_keys
+                    fn=self.write_doc_to_s3,
+                    doc=itr_doc,
+                    search_keys=key + additional_metadata,
                 )
                 for itr_doc in docs
             }
             fs, _ = wait(fs)
-            for sdoc in fs:
-                search_docs.append(sdoc.result())
+
+            search_docs = [sdoc.result() for sdoc in fs]
+
         # Use store's update to remove key clashes
-        self.index.update(search_docs)
+        self.index.update(search_docs, key=self.key)
+
+    def get_bucket(self):
+        if threading.current_thread().name == "MainThread":
+            return self.s3_bucket
+        if not hasattr(self._thread_local, "s3_bucket"):
+            session = Session(profile_name=self.s3_profile)
+            resource = session.resource("s3", endpoint_url=self.endpoint_url)
+            self._thread_local.s3_bucket = resource.Bucket(self.bucket)
+        return self._thread_local.s3_bucket
 
     def write_doc_to_s3(self, doc: Dict, search_keys: List[str]):
         """
@@ -277,7 +327,9 @@ class S3Store(Store):
             search_keys: list of keys to pull from the docs and be inserted into the
             index db
         """
-        search_doc = {k: doc[k] for k in search_keys}
+        s3_bucket = self.get_bucket()
+
+        search_doc = {k: str(doc[k]) for k in search_keys}
         search_doc[self.key] = doc[self.key]  # Ensure key is in metadata
         if self.sub_dir != "":
             search_doc["sub_dir"] = self.sub_dir
@@ -298,7 +350,7 @@ class S3Store(Store):
                 to_isoformat_ceil_ms(search_doc[self.last_updated_field])
             )
 
-        self.s3_bucket.put_object(
+        s3_bucket.put_object(
             Key=self.sub_dir + str(doc[self.key]), Body=data, Metadata=search_doc
         )
 
@@ -361,6 +413,30 @@ class S3Store(Store):
 
         self.index.update(index_docs)
 
+    def rebuild_metadata_from_index(self, index_query: dict = None):
+        """
+        Read data from the index store and populate the metadata of the S3 bucket
+        Force all of the keys to be lower case to be Minio compatible
+        Args:
+            index_query: query on the index store
+        """
+
+        qq = {} if index_query is None else index_query
+        for index_doc in self.index.query(qq):
+            key_ = self.sub_dir + index_doc[self.key]
+            s3_object = self.s3_bucket.Object(key_)
+            # make sure the keys all all lower case
+            new_meta = {str(k).lower(): v for k, v in s3_object.metadata.items()}
+            for k, v in index_doc.items():
+                new_meta[str(k).lower()] = v
+            new_meta.pop("_id")
+            # s3_object.metadata.update(new_meta)
+            s3_object.copy_from(
+                CopySource={"Bucket": self.s3_bucket.name, "Key": key_},
+                Metadata=new_meta,
+                MetadataDirective="REPLACE",
+            )
+
     def __eq__(self, other: object) -> bool:
         """
         Check equality for S3Store
