@@ -10,37 +10,38 @@ import copy
 import json
 import zlib
 from datetime import datetime
-from typing import Any, Dict, Iterator, List, Optional, Set, Tuple, Union
+from typing import Any, Dict, Iterator, List, Optional, Tuple, Union
 
 import gridfs
-from maggma.core import Sort, Store
-from maggma.stores.mongolike import MongoStore
 from monty.dev import deprecated
 from monty.json import jsanitize
 from pydash import get, has
 from pymongo import MongoClient
 
+from maggma.core import Sort, Store
+from maggma.stores.mongolike import MongoStore
+
+# https://github.com/mongodb/specifications/
+#   blob/master/source/gridfs/gridfs-spec.rst#terms
+#   (Under "Files collection document")
+files_collection_fields = (
+    "_id",
+    "length",
+    "chunkSize",
+    "uploadDate",
+    "md5",
+    "filename",
+    "contentType",
+    "aliases",
+    "metadata",
+)
+
 
 class GridFSStore(Store):
     """
     A Store for GrdiFS backend. Provides a common access method consistent with other stores
     """
 
-    # https://github.com/mongodb/specifications/
-    #   blob/master/source/gridfs/gridfs-spec.rst#terms
-    #   (Under "Files collection document")
-    files_collection_fields = (
-        "_id",
-        "length",
-        "chunkSize",
-        "uploadDate",
-        "md5",
-        "filename",
-        "contentType",
-        "aliases",
-        "metadata",
-    )
-
     def __init__(
         self,
         database: str,
@@ -50,6 +51,8 @@ class GridFSStore(Store):
         username: str = "",
         password: str = "",
         compression: bool = False,
+        ensure_metadata: bool = False,
+        searchable_fields: List[str] = [],
         **kwargs,
     ):
         """
@@ -62,6 +65,9 @@ class GridFSStore(Store):
             port: port to connec to
             username: username to connect as
             password: password to authenticate as
+            compression: compress the data as it goes into GridFS
+            ensure_metadata: ensure returned documents have the metadata fields
+            searchable_fields: fields to keep in the index store
         """
 
         self.database = database
@@ -72,8 +78,9 @@ class GridFSStore(Store):
         self.password = password
         self._collection = None  # type: Any
         self.compression = compression
+        self.ensure_metadata = ensure_metadata
+        self.searchable_fields = searchable_fields
         self.kwargs = kwargs
-        self.meta_keys = set()  # type: Set[str]
 
         if "key" not in kwargs:
             kwargs["key"] = "_id"
@@ -123,7 +130,7 @@ class GridFSStore(Store):
         """
         new_criteria = dict()
         for field in criteria:
-            if field not in cls.files_collection_fields and not field.startswith(
+            if field not in files_collection_fields and not field.startswith(
                 "metadata."
             ):
                 new_criteria["metadata." + field] = copy.copy(criteria[field])
@@ -148,7 +155,7 @@ class GridFSStore(Store):
         self,
         criteria: Optional[Dict] = None,
         properties: Union[Dict, List, None] = None,
-        sort: Optional[Dict[str, Sort]] = None,
+        sort: Optional[Dict[str, Union[Sort, int]]] = None,
         skip: int = 0,
         limit: int = 0,
     ) -> Iterator[Dict]:
@@ -160,12 +167,15 @@ class GridFSStore(Store):
         Args:
             criteria: PyMongo filter for documents to search in
             properties: properties to return in grouped documents
-            sort: Dictionary of sort order for fields
+            sort: Dictionary of sort order for fields. Keys are field names and
+                values are 1 for ascending or -1 for descending.
             skip: number documents to skip
             limit: limit on total number of documents returned
         """
         if isinstance(criteria, dict):
             criteria = self.transform_criteria(criteria)
+        elif criteria is not None:
+            raise ValueError("Criteria must be a dictionary or None")
 
         prop_keys = set()
         if isinstance(properties, dict):
@@ -179,9 +189,12 @@ class GridFSStore(Store):
             if properties is not None and prop_keys.issubset(set(doc.keys())):
                 yield {p: doc[p] for p in properties if p in doc}
             else:
+
+                metadata = doc["metadata"]
+
                 data = self._collection.find_one(
                     filter={
-                        "metadata.{}".format(self._files_store.key): doc["metadata"][
+                        "metadata.{}".format(self._files_store.key): metadata[
                             self._files_store.key
                         ]
                     },
@@ -190,7 +203,6 @@ class GridFSStore(Store):
                     sort=sort,
                 ).read()
 
-                metadata = doc["metadata"]
                 if metadata.get("compression", "") == "zlib":
                     data = zlib.decompress(data).decode("UTF-8")
 
@@ -198,13 +210,18 @@ class GridFSStore(Store):
                     data = json.loads(data)
                 except Exception:
                     pass
+
+                if self.ensure_metadata and isinstance(data, dict):
+                    data.update(metadata)
+
                 yield data
 
     def distinct(
         self, field: str, criteria: Optional[Dict] = None, all_exist: bool = False
     ) -> List:
         """
-        Get all distinct values for a field
+        Get all distinct values for a field. This function only operates
+        on the metadata in the files collection
 
         Args:
             field: the field(s) to get distinct values for
@@ -218,7 +235,7 @@ class GridFSStore(Store):
 
         field = (
             f"metadata.{field}"
-            if field not in self.files_collection_fields
+            if field not in files_collection_fields
             and not field.startswith("metadata.")
             else field
         )
@@ -230,7 +247,7 @@ class GridFSStore(Store):
         keys: Union[List[str], str],
         criteria: Optional[Dict] = None,
         properties: Union[Dict, List, None] = None,
-        sort: Optional[Dict[str, Sort]] = None,
+        sort: Optional[Dict[str, Union[Sort, int]]] = None,
         skip: int = 0,
         limit: int = 0,
     ) -> Iterator[Tuple[Dict, List[Dict]]]:
@@ -243,7 +260,8 @@ class GridFSStore(Store):
             keys: fields to group documents
             criteria: PyMongo filter for documents to search in
             properties: properties to return in grouped documents
-            sort: Dictionary of sort order for fields
+            sort: Dictionary of sort order for fields. Keys are field names and
+                values are 1 for ascending or -1 for descending.
             skip: number documents to skip
             limit: limit on total number of documents returned
 
@@ -259,7 +277,7 @@ class GridFSStore(Store):
         keys = [keys] if not isinstance(keys, list) else keys
         keys = [
             f"metadata.{k}"
-            if k not in self.files_collection_fields and not k.startswith("metadata.")
+            if k not in files_collection_fields and not k.startswith("metadata.")
             else k
             for k in keys
         ]
@@ -290,13 +308,18 @@ class GridFSStore(Store):
             bool indicating if the index exists/was created
         """
         # Transform key for gridfs first
-        if key not in self.files_collection_fields:
+        if key not in files_collection_fields:
             files_col_key = "metadata.{}".format(key)
             return self._files_store.ensure_index(files_col_key, unique=unique)
         else:
             return self._files_store.ensure_index(key, unique=unique)
 
-    def update(self, docs: Union[List[Dict], Dict], key: Union[List, str, None] = None):
+    def update(
+        self,
+        docs: Union[List[Dict], Dict],
+        key: Union[List, str, None] = None,
+        additional_metadata: Union[str, List[str], None] = None,
+    ):
         """
         Update documents into the Store
 
@@ -306,6 +329,7 @@ class GridFSStore(Store):
                  document, can be a list of multiple fields,
                  a single field, or None if the Store's key
                  field is to be used
+            additional_metadata: field(s) to include in the gridfs metadata
         """
 
         if not isinstance(docs, list):
@@ -316,11 +340,25 @@ class GridFSStore(Store):
         elif not key:
             key = [self.key]
 
-        key = list(set(key) | self.meta_keys - set(self.files_collection_fields))
+        key = list(set(key) - set(files_collection_fields))
+
+        if additional_metadata is None:
+            additional_metadata = []
+        elif isinstance(additional_metadata, str):
+            additional_metadata = [additional_metadata]
+        else:
+            additional_metadata = list(additional_metadata)
+
         for d in docs:
             search_doc = {k: d[k] for k in key}
 
-            metadata = {k: d[k] for k in [self.last_updated_field] if k in d}
+            metadata = {
+                k: get(d, k)
+                for k in [self.last_updated_field]
+                + additional_metadata
+                + self.searchable_fields
+                if has(d, k)
+            }
             metadata.update(search_doc)
             data = json.dumps(jsanitize(d)).encode("UTF-8")
             if self.compression:
@@ -349,8 +387,8 @@ class GridFSStore(Store):
             criteria = self.transform_criteria(criteria)
         ids = [cursor._id for cursor in self._collection.find(criteria)]
 
-        for id in ids:
-            self._collection.delete(id)
+        for _id in ids:
+            self._collection.delete(_id)
 
     def close(self):
         self._collection.database.client.close()
