@@ -28,7 +28,7 @@ def target():
     return store
 
 
-@pytest.fixture("module")
+@pytest.fixture(scope="module")
 def now():
     return datetime.utcnow()
 
@@ -44,7 +44,16 @@ def new_docs(now):
     return [{"lu": toc, "k": k, "v": "new"} for k in range(0, 10)]
 
 
-def test_get_items(source, target, old_docs):
+@pytest.fixture
+def some_failed_old_docs(now):
+    docs = [{"lu": now, "k": k, "v": "old", "state": "failed"} for k in range(3)]
+    docs.extend(
+        [{"lu": now, "k": k, "v": "old", "state": "failed"} for k in range(18, 20)]
+    )
+    return docs
+
+
+def test_get_items(source, target, old_docs, some_failed_old_docs):
     builder = CopyBuilder(source, target)
     source.update(old_docs)
     assert len(list(builder.get_items())) == len(old_docs)
@@ -57,6 +66,22 @@ def test_get_items(source, target, old_docs):
     assert len(list(builder.get_items())) == len(old_docs)
     assert all("v" not in d for d in builder.get_items())
 
+    source.update(some_failed_old_docs)
+    target.update(old_docs)
+    target.update(some_failed_old_docs)
+    builder = CopyBuilder(source, target)
+
+    assert len(list(builder.get_items())) == 0
+
+    builder = CopyBuilder(source, target, retry_failed=True)
+    assert len(list(builder.get_items())) == len(some_failed_old_docs)
+
+    builder = CopyBuilder(source, target, query={"k": {"$lt": 11}})
+    assert len(list(builder.get_items())) == 0
+
+    builder = CopyBuilder(source, target, retry_failed=True, query={"k": {"$lt": 11}})
+    assert len(list(builder.get_items())) == 3
+
 
 def test_process_item(source, target, old_docs):
     builder = CopyBuilder(source, target)
